==> elara-core-1.2-sources.jar: org/tools4j/elara/loop/SequencerStep.java <==
/**
 * The MIT License (MIT)
 *
 * Copyright (c) 2020 tools4j.org (Marco Terzer, Anton Anufriev)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package org.tools4j.elara.loop;

import org.tools4j.elara.input.Input;
import org.tools4j.elara.plugin.base.BaseState;
import org.tools4j.nobark.loop.Step;

import java.util.function.Function;

import static java.util.Objects.requireNonNull;

public final class SequencerStep implements Step {

    private final BaseState baseState;
    private final Input.Poller[] inputPollers;
    private final Input.Handler[] handlers;

    private int roundRobinIndex = 0;

    public SequencerStep(final BaseState baseState,
                         final Function<? super Input, ? extends Input.Handler> handlerFactory,
                         final Input... inputs) {
        this.baseState = requireNonNull(baseState);
        this.inputPollers = initPollersFor(inputs);
        this.handlers = initHandlersFor(handlerFactory, inputs);
    }

    @Override
    public boolean perform() {
        if (baseState.allEventsPolled()) {
            final int count = inputPollers.length;
            for (int i = 0; i < count; i++) {
                final int index = getAndIncrementRoundRobinIndex(count);
                if (inputPollers[index].poll(handlers[index]) > 0) {
                    return true;
                }
            }
        }
        //NOTE: we don't poll inputs during replay since
        //       (i) the command time would be difficult to define
        //      (ii) sources depending on state would operate on incomplete state
        return false;
    }

    private int getAndIncrementRoundRobinIndex(final int count) {
        final int index = roundRobinIndex;
        roundRobinIndex++;
        if (roundRobinIndex >= count) {
            roundRobinIndex = 0;
        }
        return index;
    }

    private Input.Poller[] initPollersFor(final Input... inputs) {
        final Input.Poller[] pollers = new Input.Poller[inputs.length];
        for (int i = 0; i < inputs.length; i++) {
            pollers[i] = inputs[i].poller();
        }
        return pollers;
    }

    private Input.Handler[] initHandlersFor(final Function<? super Input, ? extends Input.Handler> handlerFactory,
                                            final Input... inputs) {
        final Input.Handler[] handlers = new Input.Handler[inputs.length];
        for (int i = 0; i < inputs.length; i++) {
            handlers[i] = handlerFactory.apply(inputs[i]);
        }
        return handlers;
    }
}

==> elara-core-1.3-sources.jar: org/tools4j/elara/loop/SequencerStep.java <==
/**
 * The MIT License (MIT)
 *
 * Copyright (c) 2020 tools4j.org (Marco Terzer, Anton Anufriev)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package org.tools4j.elara.loop;

import org.tools4j.elara.input.Input;
import org.tools4j.elara.input.Receiver;
import org.tools4j.nobark.loop.Step;

import static java.util.Objects.requireNonNull;

public final class SequencerStep implements Step {

    private final Receiver receiver;
    private final Input.Poller[] inputPollers;

    private int roundRobinIndex = 0;

    public SequencerStep(final Receiver receiver, final Input... inputs) {
        this.receiver = requireNonNull(receiver);
        this.inputPollers = initPollersFor(inputs);
    }

    @Override
    public boolean perform() {
        final int count = inputPollers.length;
        for (int i = 0; i < count; i++) {
            final int index = getAndIncrementRoundRobinIndex(count);
            if (inputPollers[index].poll(receiver) > 0) {
                return true;
            }
        }
        return false;
    }

    private int getAndIncrementRoundRobinIndex(final int count) {
        final int index = roundRobinIndex;
        roundRobinIndex++;
        if (roundRobinIndex >= count) {
            roundRobinIndex = 0;
        }
        return index;
    }

    private Input.Poller[] initPollersFor(final Input... inputs) {
        final Input.Poller[] pollers = new Input.Poller[inputs.length];
        for (int i = 0; i < inputs.length; i++) {
            pollers[i] = inputs[i].poller();
        }
        return pollers;
    }
}
